
// Copyright 2019 Shlomi Steinberg
// 

#pragma once

#include "glm.hpp"
#include "common.hpp"

// Inputs:  Wavelength in nanometers
// From "Simple Analytic Approximations to the CIE XYZ Color Matching Functions"
template <typename T>
auto X_1931(T wave) {
	const auto t1 = (wave-T(442.0)) * ((wave<T(442.0))?T(0.0624):T(0.0374));
	const auto t2 = (wave-T(599.8)) * ((wave<T(599.8))?T(0.0264):T(0.0323));
	const auto t3 = (wave-T(501.1)) * ((wave<T(501.1))?T(0.0490):T(0.0382));
	return T(0.362) * glm::exp((-0.5) * sqr(t1)) + T(1.056) * glm::exp(T(-0.5) * sqr(t2)) - T(0.065) * glm::exp(T(-0.5) * sqr(t3));
}
template <typename T>
auto Y_1931(T wave) {
	const auto t1 = (wave-T(568.8)) * ((wave<T(568.8))?T(0.0213):T(0.0247));
	const auto t2 = (wave-T(530.9)) * ((wave<T(530.9))?T(0.0613):T(0.0322));
	return T(0.821) * glm::exp((-0.5) * sqr(t1)) + T(0.286) * glm::exp((-0.5) * sqr(t2));
}
template <typename T>
auto Z_1931(T wave) {
	const auto t1 = (wave-T(437.0)) * ((wave<T(437.0))?T(0.0845):T(0.0278));
	const auto t2 = (wave-T(459.0)) * ((wave<T(459.0))?T(0.0385):T(0.0725));
	return T(1.217) * glm::exp((-0.5) * sqr(t1)) + T(0.681) * glm::exp((-0.5) * sqr(t2));
}

template <typename T>
glm::vec<3,T> XYZ_1931(T wavelength) { return { X_1931(wavelength), Y_1931(wavelength), Z_1931(wavelength) }; }
