
// Copyright 2019 Shlomi Steinberg
// 

#pragma once

#include <array>
#include <complex>

namespace ior {

// Al (Aluminum)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> aluminum = {
    std::complex<fp>{ 0.417,4.447 },	// 420
    std::complex<fp>{ 0.515,4.888 },	// 460
    std::complex<fp>{ 0.626,5.321 },	// 500
    std::complex<fp>{ 0.754,5.747 },	// 540
    std::complex<fp>{ 0.909,6.165 },	// 580
    std::complex<fp>{ 1.088,6.567 },	// 620
    std::complex<fp>{ 1.304,6.942 },	// 660
    std::complex<fp>{ 1.560,7.274 },	// 700
};
// Ag (Silver)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> silver = {
	std::complex<fp>{ 0.043,2.357 },	// 420
	std::complex<fp>{ 0.041,2.777 },	// 460
	std::complex<fp>{ 0.041,3.159 },	// 500
	std::complex<fp>{ 0.043,3.522 },	// 540
	std::complex<fp>{ 0.047,3.871 },	// 580
	std::complex<fp>{ 0.050,4.209 },	// 620
	std::complex<fp>{ 0.053,4.543 },	// 660
	std::complex<fp>{ 0.055,4.870 },	// 700
};
// Au (Gold)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> gold = {
	std::complex<fp>{ 1.637,1.970 },	// 420
	std::complex<fp>{ 1.475,1.872 },	// 460
	std::complex<fp>{ 0.848,1.828 },	// 500
	std::complex<fp>{ 0.373,2.452 },	// 540
	std::complex<fp>{ 0.229,3.000 },	// 580
	std::complex<fp>{ 0.160,3.466 },	// 620
	std::complex<fp>{ 0.118,3.896 },	// 660
	std::complex<fp>{ 0.099,4.305 },	// 700
};
// Cu (Copper)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> copper = {
	std::complex<fp>{ 1.082,2.251 },	// 420
	std::complex<fp>{ 1.044,2.464 },	// 460
	std::complex<fp>{ 1.029,2.601 },	// 500
	std::complex<fp>{ 0.896,2.584 },	// 540
	std::complex<fp>{ 0.308,2.775 },	// 580
	std::complex<fp>{ 0.117,3.422 },	// 620
	std::complex<fp>{ 0.103,3.880 },	// 660
	std::complex<fp>{ 0.101,4.271 },	// 700
};
// Co (Cobalt)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> cobalt = {
	std::complex<fp>{ 1.633,3.097 },	// 420
	std::complex<fp>{ 1.772,3.339 },	// 460
	std::complex<fp>{ 1.894,3.571 },	// 500
	std::complex<fp>{ 2.024,3.775 },	// 540
	std::complex<fp>{ 2.125,3.952 },	// 580
	std::complex<fp>{ 2.194,4.121 },	// 620
	std::complex<fp>{ 2.251,4.274 },	// 660
	std::complex<fp>{ 2.305,4.434 },	// 700
};
// Cr (Chromium)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> chromium = {
	std::complex<fp>{ 2.123,2.973 },	// 420
	std::complex<fp>{ 2.411,3.185 },	// 460
	std::complex<fp>{ 2.780,3.305 },	// 500
	std::complex<fp>{ 3.103,3.330 },	// 540
	std::complex<fp>{ 3.218,3.302 },	// 580
	std::complex<fp>{ 3.164,3.303 },	// 620
	std::complex<fp>{ 3.089,3.341 },	// 660
	std::complex<fp>{ 3.054,3.386 },	// 700
};
// Fe (Iron)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> iron = {
	std::complex<fp>{ 1.048,3.974 },	// 420
	std::complex<fp>{ 1.029,4.545 },	// 460
	std::complex<fp>{ 1.079,5.150 },	// 500
	std::complex<fp>{ 1.195,5.780 },	// 540
	std::complex<fp>{ 1.374,6.434 },	// 580
	std::complex<fp>{ 1.576,7.098 },	// 620
	std::complex<fp>{ 1.908,7.811 },	// 660
	std::complex<fp>{ 2.239,8.524 },	// 700
};
// Ir (Iridium)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> iridium = {
	std::complex<fp>{ 1.667,3.770 },	// 420
	std::complex<fp>{ 1.780,4.083 },	// 460
	std::complex<fp>{ 1.924,4.371 },	// 500
	std::complex<fp>{ 2.100,4.641 },	// 540
	std::complex<fp>{ 2.276,4.892 },	// 580
	std::complex<fp>{ 2.431,5.083 },	// 620
	std::complex<fp>{ 2.560,5.274 },	// 660
	std::complex<fp>{ 2.655,5.409 },	// 700
};
// Mg (Magnesium)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> magnesium = {
	std::complex<fp>{ 0.408,3.768 },	// 420
	std::complex<fp>{ 0.535,4.191 },	// 460
	std::complex<fp>{ 0.697,4.530 },	// 500
	std::complex<fp>{ 0.784,4.851 },	// 540
	std::complex<fp>{ 0.864,5.186 },	// 580
	std::complex<fp>{ 0.949,5.534 },	// 620
	std::complex<fp>{ 1.036,5.884 },	// 660
	std::complex<fp>{ 1.133,6.244 },	// 700
};
// Ni (Nickel)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> nickel = {
	std::complex<fp>{ 1.282,3.369 },	// 420
	std::complex<fp>{ 1.113,3.894 },	// 460
	std::complex<fp>{ 1.050,4.460 },	// 500
	std::complex<fp>{ 1.070,5.033 },	// 540
	std::complex<fp>{ 1.145,5.609 },	// 580
	std::complex<fp>{ 1.240,6.186 },	// 620
	std::complex<fp>{ 1.417,6.775 },	// 660
	std::complex<fp>{ 1.593,7.363 },	// 700
};
// Fe3O4 (Iron oxide, Magnetite)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> magnetite = {
	std::complex<fp>{ 2.440,0.034 },	// 420
	std::complex<fp>{ 2.374,0.042 },	// 460
	std::complex<fp>{ 2.359,0.071 },	// 500
	std::complex<fp>{ 2.344,0.099 },	// 540
	std::complex<fp>{ 2.342,0.125 },	// 580
	std::complex<fp>{ 2.353,0.144 },	// 620
	std::complex<fp>{ 2.363,0.137 },	// 660
	std::complex<fp>{ 2.355,0.124 },	// 700
};
// Pb (Lead)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> lead = {
	std::complex<fp>{ 0.340,4.135 },	// 420
	std::complex<fp>{ 0.408,4.598 },	// 460
	std::complex<fp>{ 0.498,5.061 },	// 500
	std::complex<fp>{ 0.629,5.521 },	// 540
	std::complex<fp>{ 0.825,5.915 },	// 580
	std::complex<fp>{ 1.046,5.284 },	// 620
	std::complex<fp>{ 1.086,6.509 },	// 660
	std::complex<fp>{ 1.126,6.734 },	// 700
};
// Pt (Platinum)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> platinum = {
	std::complex<fp>{ 0.825,3.311 },	// 420
	std::complex<fp>{ 0.604,3.890 },	// 460
	std::complex<fp>{ 0.509,4.450 },	// 500
	std::complex<fp>{ 0.474,4.980 },	// 540
	std::complex<fp>{ 0.463,5.480 },	// 580
	std::complex<fp>{ 0.461,5.977 },	// 620
	std::complex<fp>{ 0.479,6.452 },	// 660
	std::complex<fp>{ 0.497,6.927 },	// 700
};
// Ta (Tantalum)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> tantalum = {
	std::complex<fp>{ 1.293,3.708 },	// 420
	std::complex<fp>{ 1.231,4.054 },	// 460
	std::complex<fp>{ 1.177,4.416 },	// 500
	std::complex<fp>{ 1.132,4.790 },	// 540
	std::complex<fp>{ 1.097,5.170 },	// 580
	std::complex<fp>{ 1.064,5.553 },	// 620
	std::complex<fp>{ 1.043,5.939 },	// 660
	std::complex<fp>{ 1.022,6.325 },	// 700
};
// Ti (Titanium)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> titanium = {
	std::complex<fp>{ 0.726,2.685 },	// 420
	std::complex<fp>{ 0.486,3.158 },	// 460
	std::complex<fp>{ 0.361,3.639 },	// 500
	std::complex<fp>{ 0.303,4.098 },	// 540
	std::complex<fp>{ 0.272,4.537 },	// 580
	std::complex<fp>{ 0.251,4.970 },	// 620
	std::complex<fp>{ 0.254,5.383 },	// 660
	std::complex<fp>{ 0.256,5.796 },	// 700
};
// TiN (titanium Nitride)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> titanium_nitride = {
	std::complex<fp>{ 1.768,0.897 },	// 420
	std::complex<fp>{ 1.408,1.026 },	// 460
	std::complex<fp>{ 1.140,1.328 },	// 500
	std::complex<fp>{ 0.995,1.648 },	// 540
	std::complex<fp>{ 0.899,1.957 },	// 580
	std::complex<fp>{ 0.838,2.258 },	// 620
	std::complex<fp>{ 0.804,2.562 },	// 660
	std::complex<fp>{ 0.790,2.845 },	// 700
};
// W (Tungsten)
template <typename fp>
static constexpr std::array<std::complex<fp>, 8> tungsten = {
	std::complex<fp>{ 1.761,5.110 },	// 420
	std::complex<fp>{ 2.212,5.073 },	// 460
	std::complex<fp>{ 2.023,4.816 },	// 500
	std::complex<fp>{ 1.480,5.147 },	// 540
	std::complex<fp>{ 1.163,5.714 },	// 580
	std::complex<fp>{ 0.933,6.373 },	// 620
	std::complex<fp>{ 0.917,7.022 },	// 660
	std::complex<fp>{ 0.900,7.671 },	// 700
};

}
